# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-09 21:39:59 +0900 (Thu, 09 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/common'

module VFS
  class Dir
    include Enumerable
    include VFS::Common
    extend  VFS::MethodGenerator
    toplevel_methods_to_instance_methods :join, :scheme, :escape, :unescape

    raise_unsupported :delete, :glob, :mkdir, :entries

    class << self
      extend  VFS::MethodGenerator
      relay_to_dir_builtin :pwd, :getwd, :chdir, :chroot

      def new(url, *args)
	# fixme; split this to common to share with VFS::File
	if self == VFS::Dir
	  scheme = VFS.scheme(url)
	  begin
	    require "vfs/scheme/#{scheme}/dir"
	  rescue LoadError
	    raise Error::Unsupported,
	      "scheme '#{scheme}' is not supported yet!"
	  end
	  handler = "::VFS::SchemeHandler::#{scheme.upcase}::Dir"
	  obj = eval(handler).send :new, url
	  unless obj
	    raise Error::Unsupported,
	      "scheme '#{scheme}' is not supported yet!"
	  end
	  return obj
	else
	  super
	end # self == VFS::Dir
      end # new

    end # class << self

    def initialize(url)
      @path    = url
      @pos     = 0
      @entries = nil
      @is_open = false
    end

    attr_accessor :is_open, :pos
    attr_reader :path
    attr_writer :entries
    private :entries=, :is_open, :is_open=
    alias :url :path
    alias :tell :pos

    alias :unlink  :delete
    alias :rmdir   :delete

    def rewind
      pos = 0
      self
    end

    def seek(p)
      pos = p
      self
    end

    def each(&block)
      entries.each(&block)
    end
    alias :foreach :each

    def read
      c = entries[pos]
      pos += 1
      return c
    end

    def open(*arg, &block)
      is_open = true
      yield(self)
    end

    def close
      is_open = flase
      log("(BUG?) Should override close().", Log::CRIT)
      nil
    end

    def [](pat)
      glob(pat)
    end

    instance_methods_to_class_methods(
      *self.instance_methods(false).delete_if{|m| m =~ /=$/})
  end # Dir
end # VFS
