# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-09 21:39:59 +0900 (Thu, 09 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/dir'
require 'vfs/scheme/http/common'

module VFS;  module SchemeHandler;  module HTTP;
  class Dir < VFS::Dir
    include VFS::SchemeHandler::HTTP::Common
    
    def initialize(url)
      check_uri_syntax(url)
      super
      init_common_attr
    end

    def entries
      @entries unless @entries == nil
      base = parse_path(url)
      if capability? "PROPFIND"
	http_request(url, :propfind, nil, {"Depth" => "1"}) { |rep|
	  @entries = []
	  rep.body.scan(/<D:href(?:\s+[^>]*)?>(.*?)<\/D:href>/m) { |e, |
	    # FIXME!!! MUST UNESCAPE XML ENTITY.
	    log("*BUG/NotInplementedYet* must unescape XML entity.", Log::WARNING)
	    e = e.delete("\n\r \t").sub(/^#{Regexp.quote(base)}/, '')
	    e == "" and next
	    @entries << VFS.escape(e)
	  }
	}
      else
	http_request(url) { |rep|
	  @entries = []
	  rep.body.scan(/<a href=\"([^"]+)\"/) {|h, |
	    # fixme. VERY ad-hoc way...
	    h.sub!(/^\.\//, '')
	    h == ""           and next
	    h =~ /^\?/        and next
	    h =~ /^\//        and next # OK? realy??
	    h =~ /^\.\.\//    and next
	    h =~ /^[a-z+-]+:/ and next
	    # FIXME!!! MUST UNESCAPE HTML ENTITY.
	    log("*BUG/NotInplementedYet* must unescape HTML entity.", Log::WARNING)
	    if h =~ /\/(?!$)/
	      # Hmmmmmmmmmmm.
	      # fixme? User can open any URL resources as Dir.open,
	      # but script can't detect directory or not.
	      # may I raise exception like as Errno::ENOTDIR here.
	      raise Error::Unknown,
		"A directory entory ('#{h}') has '/'. This is bug."
	    end
	    @entries << VFS.escape(h)
	  }
	}
      end
      @entries
    end

  end
end; end; end

if $0 == __FILE__

#  url = "http://localhost/~sugi/"
  url = "http://sugi.nemui.org/pub/prog/"
  f = VFS::SchemeHandler::HTTP::Dir.new(url)
  p f.capability
  p f.entries

end
