# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-08 22:10:32 +0900 (Wed, 08 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/override'
require 'zip/zip'

module VFS;  module SchemeHandler;
  module ZIP;
    class File < VFS::File
      # TODO :utime, :delete, :unlink, :rename, :truncate,
      # :lstat :stat write-open, and many...........

      def initialize(url)
	unless Regexp.new("^zip:").match(url)
	  raise Error::InvalidURI,
	    "invalid zip URL '#{url}' was speficied for zip scheme handler"
	end
	super
	if Regexp.new("^zip:([^!]+)!(.+)$").match(url)
	  @ziparc  = $1
	  @zipcont = $2
	elsif Regexp.new("^zip:(.*?/[^/]+\.(?i:zip))/(.+)$").match(url)
	  @ziparc  = $1
	  @zipcont = $2
	else
	  raise Error::InvalidURI,
	    "can't determin zip file path"
	end
	@zipentry  = nil
      end
      attr_reader :ziparc, :zipcont
      private :ziparc, :zipcont

      def ftype
	"file(zip)"
      end

      def exists?(*args)
	begin 
	  zipentry
	rescue Errno::ENOENT
	  return false
	end
	true
      end
      alias :exist? :exists?

      def zipentry
	if @zipentry == nil
	  zf = Zip::ZipFile.open(ziparc)
	  # FIXME!!!! check file or directory!
	  @zipentry = zf.find_entry(zipcont)
	  unless @zipentry
	    raise Errno::ENOENT
	  end
	  zf.close
	end
	@zipentry
      end
      private :zipentry
      
      def mtime
	zipentry.time
      end
      alias :ctime :mtime
      alias :atime :mtime

      def size
	zipentry.size
      end

      def open_for_read(*args, &block)
	stream = zipentry.get_input_stream
	class << stream
	  def sysread(len, *buf)
	    t = read(len)
	    raise EOFError if t == nil
	    if buf.length > 0
	      raise Error::Unsupported,
		"sysread emulation: direct input to buf has not been supported"
	    end
	    t
	  end
	end
	block_given? ? yield(stream) : stream
      end

      def open_for_write(*args, &block)
	super # FXIME. only fail........
      end

#      ["chardev?", "zero?", "writable_real?", "pipe?", "exists?", "sticky?", "file?", "blockdev?", "writable?", "grpowned?", "exist?", "setgid?", "executable_real?", "socket?", "readable_real?", "owned?", "directory?", "setuid?", "executable?", "symlink?", "size", "readable?", "size?" ]

    end # VFS::SchemeHandler::ZIP::File
end; end; end


if __FILE__ == $0
#  f = "zip:/home/sugi/data/mail/local/tosc-00000.zip/1"
  f = "zip:http://localhost/~sugi/test.zip/1"
  vfs = VFS::SchemeHandler::ZIP::File.new(f)
  p vfs
  p VFS::scheme(f)
  p vfs.scheme
  p vfs.mtime
  p vfs.size
  p vfs.basename
  p vfs.dirname
  p vfs.scheme
  io = vfs.open
  puts io.read(20)
  io.close
  vfs.open { |io| puts io.read(20) }
end
